@@ -16,7 +16,7 @@ Copyright (c) Intel Corporation (2009-2017).
 
 \*****************************************************************************/
 
-#include "common_clang.h"
+#include "opencl_clang.h"
 #include "options.h"
 
 #include "clang/Driver/Options.h"
@@ -31,7 +31,10 @@ Copyright (c) Intel Corporation (2009-2017).
 #include <map>
 #include <sstream>
 
-#define PREFIX(NAME, VALUE) const char *const NAME[] = VALUE;
+#define PREFIX(NAME, VALUE)                                                    \
+  static constexpr llvm::StringLiteral NAME##_init[] = VALUE;                  \
+  static constexpr llvm::ArrayRef<llvm::StringLiteral> NAME(                   \
+      NAME##_init, std::size(NAME##_init) - 1);
 #define OPTION(PREFIX, NAME, ID, KIND, GROUP, ALIAS, ALIASARGS, FLAGS, PARAM,  \
                HELPTEXT, METAVAR, VALUES)
 #include "opencl_clang_options.inc"
@@ -40,14 +43,16 @@ Copyright (c) Intel Corporation (2009-2017).
 
 using namespace llvm::opt;
 
-static const OptTable::Info ClangOptionsInfoTable[] = {
+extern llvm::ManagedStatic<llvm::sys::SmartMutex<true>> compileMutex;
+
+static constexpr OptTable::Info ClangOptionsInfoTable[] = {
 #define PREFIX(NAME, VALUE)
 #define OPTION(PREFIX, NAME, ID, KIND, GROUP, ALIAS, ALIASARGS, FLAGS, PARAM,  \
                HELPTEXT, METAVAR, VALUES)                                      \
   {                                                                            \
     PREFIX, NAME, HELPTEXT, METAVAR, OPT_COMPILE_##ID,                         \
         llvm::opt::Option::KIND##Class, PARAM, FLAGS, OPT_COMPILE_##GROUP,     \
-        OPT_COMPILE_##ALIAS, ALIASARGS                                         \
+        OPT_COMPILE_##ALIAS, ALIASARGS, VALUES                                 \
   }                                                                            \
   ,
 #include "opencl_clang_options.inc"
@@ -58,70 +63,6 @@ OpenCLCompileOptTable::OpenCLCompileOptTable()
 
 int EffectiveOptionsFilter::s_progID = 1;
 
-// This code was adopted from the SPIRV-LLVM-Translator repository.
-static int parseSPVExtOption(
-    llvm::StringRef SPVExt,
-    SPIRV::TranslatorOpts::ExtensionsStatusMap &ExtensionsStatus) {
-  // Map name -> id for known extensions
-  std::map<llvm::StringRef, SPIRV::ExtensionID> ExtensionNamesMap;
-#define _STRINGIFY(X) #X
-#define STRINGIFY(X) _STRINGIFY(X)
-#define EXT(X) ExtensionNamesMap[STRINGIFY(X)] = SPIRV::ExtensionID::X;
-#include "LLVMSPIRVExtensions.inc"
-#undef EXT
-#undef STRINGIFY
-#undef _STRINGIFY
-
-  // Set the initial state: assume that any known extension is disallowed.
-  llvm::Optional<bool> DefaultVal;
-  for (const auto &It : ExtensionNamesMap)
-    ExtensionsStatus[It.second] = DefaultVal;
-
-  llvm::SmallVector<llvm::StringRef, 32> SPVExtList;
-  llvm::SplitString(SPVExt, SPVExtList, ",");
-
-  if (SPVExtList.empty()) {
-    llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
-                 << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
-    return -1;
-  }
-
-  for (unsigned i = 0; i < SPVExtList.size(); ++i) {
-    llvm::StringRef ExtString = SPVExtList[i];
-    if (ExtString.empty() ||
-        ('+' != ExtString.front() && '-' != ExtString.front())) {
-      llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
-                   << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
-      return -1;
-    }
-
-    auto ExtName = ExtString.substr(1);
-
-    if (ExtName.empty()) {
-      llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
-                   << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
-      return -1;
-    }
-
-    bool ExtStatus = ('+' == ExtString.front());
-    if ("all" == ExtName) {
-      // Update status for all known extensions
-      for (const auto &It : ExtensionNamesMap)
-        ExtensionsStatus[It.second] = ExtStatus;
-    } else {
-      // Ignore unknown extensions, as targets may support non-standard SPIR-V
-      // extensions. Do not reject them; this approach is more tolerant.
-      const auto &It = ExtensionNamesMap.find(ExtName);
-      if (ExtensionNamesMap.end() == It)
-        continue;
-
-      ExtensionsStatus[It->second] = ExtStatus;
-    }
-  }
-
-  return 0;
-}
-
 ///
 // Options filter that validates the opencl used options
 //
@@ -233,14 +174,18 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
       // default:
       // assert(false && "some unknown argument");
     case OPT_COMPILE_profiling:
-    case OPT_COMPILE_g_Flag:
-      effectiveArgs.push_back("-debug-info-kind=limited");
-      effectiveArgs.push_back("-dwarf-version=4");
-      break;
     case OPT_COMPILE_gline_tables_only_Flag:
       effectiveArgs.push_back("-debug-info-kind=line-tables-only");
       effectiveArgs.push_back("-dwarf-version=4");
       break;
+    case OPT_COMPILE_g_Flag:
+      effectiveArgs.push_back("-debug-info-kind=limited");
+      effectiveArgs.push_back("-dwarf-version=4");
+#ifdef _WIN32
+      // Do not use column information on Windows.
+      effectiveArgs.push_back("-gno-column-info");
+#endif
+      break;
     }
   }
 
@@ -298,7 +243,7 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
   // OpenCL v2.0 s6.9.u - Implicit function declaration is not supported.
   // Behavior of clang is changed and now there is only warning about
   // implicit function declarations. To be more user friendly and avoid
-  // unexpected indirect function calls in IR, let's force this warning to
+  // unexpected indirect function calls in BE, let's force this warning to
   // error.
   effectiveArgs.push_back("-Werror=implicit-function-declaration");
 
@@ -308,10 +253,30 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
 
   for (auto it = effectiveArgs.begin(), end = effectiveArgs.end(); it != end;
        ++it) {
-    if (it->compare("-Dcl_khr_fp64") == 0)
+    if (it->compare("-Dcl_khr_fp64") == 0 || it->compare("-D cl_khr_fp64=1") == 0)
       fp64Enabled = true;
+    else if (it->compare("-U cl_khr_fp64") == 0)
+      fp64Enabled = false;
+    // Find last position that enables or disables cl_khr_fp64
+    else if (it->find("cl_khr_fp64") != std::string::npos) {
+      auto NegFp64 = it->rfind("-cl_khr_fp64");
+      auto PosFp64 = it->rfind("+cl_khr_fp64");
+      if(NegFp64 != std::string::npos && PosFp64 != std::string::npos)
+         fp64Enabled = PosFp64 > NegFp64;
+      else if(NegFp64 != std::string::npos)
+        fp64Enabled = false;
+      else
+        fp64Enabled = true;
+    }
   }
 
+#ifdef PCH_EXTENSION
+  std::map<std::string, bool> extMap;
+  llvm::SmallVector<llvm::StringRef> extVec;
+  llvm::SplitString(PCH_EXTENSION, extVec, ",");
+  for(auto ext : extVec)
+    extMap.insert({ext.str(), true});
+#else
   std::map<std::string, bool> extMap{
       {"cl_khr_3d_image_writes", true},
       {"cl_khr_depth_images", true},
@@ -333,6 +298,7 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
       {"cl_intel_planar_yuv", true},
       {"cl_intel_subgroups", true},
       {"cl_intel_subgroups_short", true}};
+#endif
 
   auto parseClExt = [&](const std::string &clExtStr) {
     llvm::StringRef clExtRef(clExtStr);
@@ -429,8 +395,8 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
   return sourceName;
 }
 
-int CompileOptionsParser::processOptions(const char *pszOptions,
-                                         const char *pszOptionsEx) {
+void CompileOptionsParser::processOptions(const char *pszOptions,
+                                          const char *pszOptionsEx) {
   // parse options
   unsigned missingArgIndex, missingArgCount;
   std::unique_ptr<OpenCLArgList> pArgs(
@@ -444,27 +410,16 @@ int CompileOptionsParser::processOptions(const char *pszOptions,
   for (ArgsVector::iterator it = m_effectiveArgs.begin(),
                             end = m_effectiveArgs.end();
        it != end; ++it) {
-    llvm::StringRef arg(*it);
-    (void)arg.consume_front("-");
-    // support -- prefix as well.
-    (void)arg.consume_front("-");
-    if (arg == "cl-opt-disable") {
+    if (it->compare("-cl-opt-disable") == 0) {
       m_optDisable = true;
-    } else if (arg == "emit-spirv") {
+    }
+    else if (it->compare("-emit-spirv") == 0) {
       m_effectiveArgsRaw.push_back("-emit-llvm-bc");
       m_emitSPIRV = true;
       continue;
-    } else if (arg.consume_front("spirv-ext=")) {
-      m_hasSPIRVExt = true;
-      // m_SPIRVExtStatusMap will be initialized and updated according to `arg`.
-      int ret = parseSPVExtOption(arg, m_SPIRVExtStatusMap);
-      if (0 != ret)
-        return ret;
-      continue;
     }
     m_effectiveArgsRaw.push_back(it->c_str());
   }
-  return 0;
 }
 
 bool CompileOptionsParser::checkOptions(const char *pszOptions,
@@ -516,6 +471,10 @@ std::string CompileOptionsParser::getEffectiveOptionsAsString() const {
 extern "C" CC_DLL_EXPORT bool CheckCompileOptions(const char *pszOptions,
                                                   char *pszUnknownOptions,
                                                   size_t uiUnknownOptionsSize) {
+  // LLVM doesn't guarantee thread safety,
+  // therefore we serialize execution of LLVM code.
+  llvm::sys::SmartScopedLock<true> compileOptionsGuard{*compileMutex};
+
   try {
     CompileOptionsParser optionsParser("200");
     return optionsParser.checkOptions(pszOptions, pszUnknownOptions,
